import { world, system } from "@minecraft/server";

// Map to store active TNT entities along with their last known locations
const activeTNTs = new Map();

// Radius within which to apply the camera shake effect (in blocks)
const SHAKE_RADIUS = 5;

// Function to calculate the Euclidean distance between two locations
function calculateDistance(loc1, loc2) {
  return Math.sqrt(
    Math.pow(loc1.x - loc2.x, 2) +
    Math.pow(loc1.y - loc2.y, 2) +
    Math.pow(loc1.z - loc2.z, 2)
  );
}

// Subscribe to entity spawn events to track TNT entities when they appear
world.afterEvents.entitySpawn.subscribe(event => {
  const entity = event.entity;
  if (entity.typeId === "minecraft:tnt") {
    // Store the TNT's ID and its current location
    activeTNTs.set(entity.id, entity.location);
  }
});

// Subscribe to entity move events to update TNT locations as they move
world.afterEvents.entityMove.subscribe(event => {
  const entity = event.entity;
  if (activeTNTs.has(entity.id)) {
    // Update the last known location of the TNT
    activeTNTs.set(entity.id, entity.location);
  }
});

// Subscribe to entity removal events to detect TNT explosions
world.afterEvents.entityRemove.subscribe(event => {
  const entityId = event.entityId;
  if (activeTNTs.has(entityId)) {
    // Retrieve the last known location of the TNT, which indicates the explosion point
    const explosionLocation = activeTNTs.get(entityId);
    // Remove the TNT from our tracking map
    activeTNTs.delete(entityId);

    // Loop through all players in the world
    for (const player of world.getPlayers()) {
      // Calculate the distance from the player to the explosion location
      const distance = calculateDistance(player.location, explosionLocation);
      if (distance <= SHAKE_RADIUS) {
        // If the player is within the shake radius, run the shake function for that player
        system.run(() => {
          player.runCommandAsync("function shake").catch(err => {
            console.error(`Error executing function shake for player ${player.name}:`, err);
          });
        });
      }
    }
  }
});